<?php
/**
 * This class is used to enable self-hosted updates for the plugin.
 *
 * @package interlinks-manager
 */

/**
 * This class is used to enable self-hosted updates for the plugin.
 */
class Daim_Update_Checker {

	/**
	 * The singleton instance of the class.
	 *
	 * @var null
	 */
	protected static $instance = null;

	/**
	 * The instance of the shared class.
	 *
	 * @var Daim_Shared|null
	 */
	private $shared = null;

	/**
	 * Constructor.
	 */
	public function __construct() {

		// Assign an instance of the plugin info.
		$this->shared = Daim_Shared::get_instance();

		/**
		 * Filters the response for the current WordPress.org Plugin Installation API request performed with
		 * the plugins_api() method.
		 *
		 * - https://developer.wordpress.org/reference/functions/plugins_api/
		 *
		 * This filter os used to provide the plugin information for our custom plugin. More in details, by modifying
		 * the result of the "plugins_api" filter callback, we can include the plugin information for our custom plugin.
		 *
		 * This filter runs for example when the user clicks on the "View version x.xx details." link in the Plugins
		 * menu of the WordPress dashboard.
		 *
		 * ## When is the "plugins_api" filter Used?
		 *
		 * The plugins_api filter is involved when WordPress fetches information about a plugin, such as:
		 *
		 * - Searching for plugins on the Plugins > Add New screen.
		 * - Displaying details about a plugin (e.g., clicking "More Details").
		 * - Showing lists of plugins (e.g., popular, recommended, or featured).
		 *
		 * Ref: https://developer.wordpress.org/reference/hooks/plugins_api/
		 */
		add_filter( 'plugins_api', array( $this, 'info' ), 20, 3 );

		/**
		 * Filter the "update_plugins" transient to report your plugin as out of date.
		 *
		 * If an update is available, the notification, with included a link to update the plugin, will be displayed in
		 * the WordPress dashboard.
		 *
		 * Ref: https://gist.github.com/danielbachhuber/7684646
		 *
		 * When the Filter Runs:
		 *
		 * - The pre_set_site_transient_update_plugins filter is applied when WordPress is about to save the result of
		 * the plugin update check into the site_transient for plugin updates (update_plugins).
		 * - This transient contains information about all installed plugins and whether there are updates available.
		 * - The transient is cached and only updated when a scheduled update check runs, or when manually triggered
		 * (e.g., via the "Check for updates" button in the dashboard).
		 *
		 * Note that the plugin information added with this filter are also used to perform the update and download of
		 * the custom plugin for example when the user clicks on the "update now" link in the Plugins menu of the WordPress
		 * dashboard.
		 */
		add_filter( 'site_transient_update_plugins', array( $this, 'update' ) );

		/**
		 * Fires when the upgrader process is complete.
		 *
		 * The "upgrader_process_complete" action hook is run when the download process for a plugin install or update
		 * finishes.
		 *
		 * Use with caution: When you use the upgrader_process_complete action hook in your plugin and your plugin is
		 * the one which under upgrade, then this action will run the old version of your plugin.
		 *
		 * In this case the filter is used to add the action to delete the transient used to store the plugin update
		 * information.
		 *
		 * Ref: https://developer.wordpress.org/reference/hooks/upgrader_process_complete/
		 */
		add_action( 'upgrader_process_complete', array( $this, 'purge' ), 10, 2 );
	}

	/**
	 * Create a singleton instance of the class.
	 *
	 * @return self|null
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Include plugin information for our custom plugin using the callback of the "plugins_api" filter.
	 *
	 * The plugin information for Interlinks Manager is included when:
	 *
	 * - The $action is set to "plugin_information".
	 * - The $arg->slug is equal to "interlinks-manager/init.php".
	 *
	 * Note that returning a non-false value will effectively short-circuit the WordPress.org API request.
	 *
	 * @param false|object|array $res This is the current result or false by default. You can modify it to return custom data instead of letting the plugins_api() function proceed.
	 * @param string             $action The action being requested.
	 * @param object             $arg The arguments passed to plugins_api() that control the details of the request. These can include the plugin slug, search terms, etc.
	 *
	 * @return mixed|stdClass
	 */
	public function info( $res, $action, $arg ) {

		// Do nothing if you're not getting plugin information right now.
		if ( 'plugin_information' !== $action ) {
			return $res;
		}

		// Do nothing if it is not this plugin.
		if ( ! isset( $arg->slug ) || 'interlinks-manager' !== $arg->slug ) {
			return $res;
		}

		// Fetch the plugin information from the remote server.
		$remote = $this->shared->fetch_remote_plugin_info();

		/**
		 * Return the unmodified response under specific conditions.
		 */
		if ( ! $this->response_is_valid( $remote ) ) {
			return $res;
		}

		$res = new stdClass();

		$res->name           = $remote->name;
		$res->slug           = $remote->slug;
		$res->version        = $remote->version;
		$res->tested         = $remote->tested;
		$res->requires       = $remote->requires;
		$res->author         = $remote->author;
		$res->author_profile = $remote->author_profile;
		$res->requires_php   = $remote->requires_php;
		$res->last_updated   = $remote->last_updated;

		$res->sections = array(
			'changelog' => $remote->sections->changelog,
		);

		if ( ! empty( $remote->banners ) ) {
			$res->banners = array(
				'low'  => $remote->banners->low,
				'high' => $remote->banners->high,
			);
		}

		return $res;
	}

	/**
	 * Report the plugin as out of date if needed by adding information to the transient.
	 *
	 * @param array $transient This is a transient data that contains the information about the plugin updates required.
	 */
	public function update( $transient ) {

		// Check if the transient already contains an update for our plugin.
		if ( empty( $transient->checked ) ) {
			return $transient;
		}

		// Replace 'your-plugin/your-plugin.php' with your plugin’s folder and main file name.
		$plugin_slug     = 'interlinks-manager/init.php';
		$current_version = $transient->checked[ $plugin_slug ];

		// Fetch the plugin information from the remote server.
		$data = $this->shared->fetch_remote_plugin_info();

		/**
		 * Return the unmodified response under specific conditions.
		 */
		if ( ! $this->response_is_valid( $data ) ) {
			return $data;
		}

		if ( version_compare( $current_version, $data->version, '<' ) ) {

			// Prepare the plugin update data.
			$transient->response[ $plugin_slug ] = (object) array(
				'slug'        => $data->slug,
				'plugin'      => $plugin_slug,
				'new_version' => $data->version,
				'package'     => $data->download_url,
				'banners' => array(
					'1x'  => $data->banners->low,
					'2x' => $data->banners->high,
				),
				'requires'    => $data->requires,
				'tested'      => $data->tested,
				'requires_php' => $data->requires_php,
			);

		}

		return $transient;
	}

	/**
	 * Delete the transient used to store the plugin update information when the new plugin version is installed.
	 *
	 * @param WP_Upgrader $upgrader The WP_Upgrader instance.
	 * @param array       $options The array of options for the upgrader.
	 */
	public function purge( $upgrader, $options ) {

		if (
			'update' === $options['action']
			&& 'plugin' === $options['type']
		) {
			// Clean the cache when new plugin version is installed.
			delete_transient( DAIM_WP_PLUGIN_UPDATE_INFO_TRANSIENT );
		}
	}

	/**
	 * Check if the response is valid.
	 *
	 * @param object $response The response object.
	 *
	 * @return bool
	 */
	public function response_is_valid( $response ) {

		if (
			( isset( $response->error ) && 'invalid_license' === $response->error ) ||
			is_wp_error( $response ) ||
			false === $response
		) {
			return false;
		} else {
			return true;
		}
	}
}
